import 'package:eyflutter_cmi/collect/event/on_event_collect.dart';
import 'package:eyflutter_cmi/enums/sdk_platform.dart';
import 'package:eyflutter_core/eyflutter_core.dart';

class EventCollect with OnEventCollect {
  String _eventAction = "0ff4c8a837fd8d44";

  @override
  void eventObject(
      {required SDKPlatform platform, required String eventId, Map<String, dynamic> arguments = const {}}) {
    if (!_argumentsCheck(eventId, arguments)) {
      return;
    }
    arguments["platform"] = platform.name;
    arguments["eventId"] = eventId;
    CloudChannelManager.instance.send(_eventAction, arguments: arguments);
  }

  List<String> _ruleOutAttributes() {
    return [
      "id",
      "ts",
      "du",
      "ds",
      "duration",
      "pn",
      "token",
      "device_name",
      "device_model",
      "device_brand",
      "country",
      "city",
      "channel",
      "province",
      "appkey",
      "app_version",
      "access",
      "launch",
      "pre_app_version",
      "terminate",
      "no_first_pay",
      "is_newpayer",
      "first_pay_at",
      "first_pay_level",
      "first_pay_source",
      "first_pay_user_level",
      "first_pay_version",
      "type"
    ];
  }

  bool _argumentsCheck(String eventId, Map<String, dynamic> map) {
    if (map.length > 256) {
      Logger.instance.error("自定义事件$eventId的参数上线为256个,上报失败.");
      return false;
    }
    var outAttributes = _ruleOutAttributes();
    Set<String> keepFields = {};
    Set<String> lenLimitFields = {};
    map.forEach((key, value) {
      if (outAttributes.contains(key)) {
        keepFields.add(key);
      } else if (key.startsWith("_")) {
        keepFields.add(key);
      }
      if ((value is List) && value.length > 100) {
        lenLimitFields.add(key);
      }
    });
    if (!_keepAttrs(keepFields)) {
      return false;
    }
    return _limitAttrs(lenLimitFields);
  }

  bool _limitAttrs(Set<String> lenLimitFields) {
    int len = lenLimitFields.length;
    if (len > 0) {
      var limitFields = StringBuffer();
      int pos = 0;
      lenLimitFields.forEach((element) {
        limitFields.write("$element");
        if ((pos + 1) < len) {
          limitFields.write(",");
        }
        pos++;
      });
      Logger.instance.error("参数属性[${limitFields.toString()}]数组大小不能超过100");
      return false;
    }
    return true;
  }

  bool _keepAttrs(Set<String> keepFields) {
    int len = keepFields.length;
    if (len > 0) {
      var warningFields = StringBuffer();
      int pos = 0;
      keepFields.forEach((element) {
        warningFields.write("$element");
        if ((pos + 1) < len) {
          warningFields.write(",");
        }
        pos++;
      });
      Logger.instance.error("参数属性[${warningFields.toString()}]为保留字段且不能以'_'开头");
      return false;
    }
    return true;
  }
}
