import 'dart:io';

import 'package:eyflutter_cmi/collect/event/on_collect_initialize.dart';
import 'package:eyflutter_cmi/enums/sdk_platform.dart';
import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:flutter/material.dart';

class Initialize with OnCollectInitialize {
  String _collectInitAction = "1de4a91acbe275ef";

  //初始化状态缓存
  String _initStateKey = "00e4e59a39ab79c5";

  /// 需要进行设备信息、日志等数据收集时在引擎启动时初始化
  /// [platform] 根据对应的平台初始化
  /// system:实现systemInitCall回调由用户自行处理;
  /// umeng:初始化umeng sdk
  @override
  void init(SDKPlatform platform, {VoidCallback? systemInitCall}) async {
    var config = EyFlutter.instance.appConfig;
    var appKey = config?.appKey;
    var appInfo = await config?.appInfo();
    if (![SDKPlatform.system, SDKPlatform.eyinfo].contains(platform)) {
      var sdkAppKey = await _getSdkAppKey(appKey, platform);
      if (sdkAppKey.isEmptyString) {
        return;
      }
      _thirdSdkInit(platform, sdkAppKey, appInfo);
    } else {
      if (platform == SDKPlatform.system) {
        if (systemInitCall != null) {
          systemInitCall();
        }
      } else if (platform == SDKPlatform.eyinfo) {
        _eySdkInit();
      }
    }
  }

  void _eySdkInit() async {
    var initState = await MmkvUtils.instance.getBool(_initStateKey, part: MMkvPart.state);
    if (!(initState ?? false)) {
      var deviceInfo = await DeviceUtils.instance.deviceInfo;
      var url = (await EyFlutter.instance.addressConfig?.deviceUrl()) ?? "";
      Retrofit retrofit = Retrofit(api: url, method: Method.POST);
      DioManager.instance.request(
          retrofit: retrofit,
          params: {
            "deviceId": deviceInfo.deviceId,
            "deviceType": deviceInfo.deviceType,
            "manufacturer": deviceInfo.manufacturer,
            "brand": deviceInfo.brand,
            "model": deviceInfo.model,
            "board": deviceInfo.board,
            "hardware": deviceInfo.hardware,
            "host": deviceInfo.host,
            "display": deviceInfo.display,
            "density": deviceInfo.density,
            "sys": deviceInfo.sys,
            "abis": deviceInfo.abis,
            "root": deviceInfo.root,
            "simulator": deviceInfo.simulator,
            "extension": deviceInfo.extension
          },
          successCall: (String requestId, Map<String, dynamic> dataMap) {
            MmkvUtils.instance.putBool(_initStateKey, part: MMkvPart.state, value: true);
          },
          errorCall: (String requestId, String message, Map<String, dynamic> dataMap) {
            print(message);
          });
    }
  }

  void _thirdSdkInit(SDKPlatform platform, String sdkAppKey, ApplicationInfo? appInfo) {
    CloudChannelManager.instance.send(_collectInitAction, arguments: {
      "platform": platform.name,
      "appKey": sdkAppKey,
      "channel": appInfo?.channel,
    });
  }

  Future<String> _getSdkAppKey(String? eyAppKey, SDKPlatform platform) async {
    String cacheKey = "${eyAppKey}_${platform.name}";
    var platAppKey = await MmkvUtils.instance.getString(cacheKey);
    if (platAppKey.isNotEmptyString) {
      return Future.value(platAppKey);
    }
    //获取key配置url
    var url = (await EyFlutter.instance.addressConfig?.getSdkKeyApiUrl()) ?? "";
    //请求对应平台key
    Retrofit retrofit = Retrofit(api: url, method: Method.GET);
    Map<String, dynamic> data = await DioManager.instance.asyncRequest(retrofit: retrofit, params: {
      "clientKey": eyAppKey,
      "platCode": platform.name,
      "deviceType": Platform.isAndroid ? 1 : (Platform.isIOS ? 2 : 0)
    });
    String ak = data["appKey"] ?? "";
    if (ak.isNotEmptyString) {
      MmkvUtils.instance.putString(cacheKey, value: ak);
    }
    return ak;
  }
}
