import 'dart:io';

import 'package:eyflutter_cmi/collect/collector/event_collect.dart';
import 'package:eyflutter_cmi/collect/collector/initialize.dart';
import 'package:eyflutter_cmi/collect/collector/page_collect.dart';
import 'package:eyflutter_cmi/collect/collector/profile_collect.dart';
import 'package:eyflutter_cmi/collect/event/on_collect_initialize.dart';
import 'package:eyflutter_cmi/collect/event/on_page_collect.dart';
import 'package:eyflutter_cmi/collect/event/on_profile_collect.dart';
import 'package:eyflutter_cmi/enums/sdk_platform.dart';
import 'package:eyflutter_core/eyflutter_core.dart';

class DataCollect {
  factory DataCollect() => _getInstance();

  static DataCollect get instance => _getInstance();
  static DataCollect? _instance;

  static DataCollect _getInstance() {
    return _instance ??= new DataCollect._internal();
  }

  DataCollect._internal();

  String _killAction = "2b84715914c0f771";

  /// 数据收集初始化，一般在应用启动时调用
  /// [platforms] 要初始化的平台
  void init({required List<SDKPlatform> platforms}) {
    OnCollectInitialize collectInitialize = Initialize();
    platforms.forEach((element) {
      collectInitialize.init(element);
    });
  }

  /// 应用退出前调用用于保存统计数据
  void killProcess({required SDKPlatform platform}) {
    if (Platform.isAndroid) {
      CloudChannelManager.instance.send(_killAction, arguments: {
        "platform": platform.name,
      });
    }
  }

  /// 登录统计(一般在用户登录时调用)
  /// [account] 用户账号(相对用户唯一)
  /// [provider] 账号来源,表示用于区分第三方平台标识
  void profileSignin({required SDKPlatform platform, required String account, String provider = ""}) {
    OnProfileCollect profileCollect = ProfileCollect();
    profileCollect.profileSignin(platform: platform, account: account, provider: provider);
  }

  /// 退出登录统计(一般在用户退出时调用)
  void profileSignOff({required SDKPlatform platform}) {
    OnProfileCollect profileCollect = ProfileCollect();
    profileCollect.profileSignOff(platform: platform);
  }

  /// 页面进入时调用
  /// [viewName] 页面名称
  void pageStart({required SDKPlatform platform, required String viewName}) {
    OnPageCollect pageCollect = PageCollect();
    pageCollect.pageStart(platform: platform, viewName: viewName);
  }

  /// 页面退出时调用
  /// [viewName] 页面名称
  void pageEnd({required SDKPlatform platform, required String viewName}) {
    OnPageCollect pageCollect = PageCollect();
    pageCollect.pageEnd(platform: platform, viewName: viewName);
  }

  /// 事件统计
  /// [eventId] 自定义事件ID，指一个操作
  /// [arguments] 事件统计参数
  /// key：自定义事件下的参数，一个可选的描述符，用于定义事件操作的属性信息
  /// value：自定义事件参数下的参数值，用于描述事件属性的详细信息。
  void eventObject(
      {required SDKPlatform platform, required String eventId, Map<String, dynamic> arguments = const {}}) {
    EventCollect eventCollect = EventCollect();
    eventCollect.eventObject(platform: platform, eventId: eventId, arguments: arguments);
  }
}
