import 'dart:io';

import 'package:eyflutter_cmi/update/beans/version_info.dart';
import 'package:eyflutter_cmi/update/event/on_app_update.dart';
import 'package:eyflutter_core/eyflutter_core.dart';

class UpdateManager {
  factory UpdateManager() => _getInstance();

  static UpdateManager get instance => _getInstance();
  static UpdateManager? _instance;

  static UpdateManager _getInstance() {
    return _instance ??= new UpdateManager._internal();
  }

  UpdateManager._internal() {
    _isUpdating = false;
  }

  String _timeCacheKey = "a084eeaa5ad33c0c";
  String _updateConfigKey = "9f649d09e18c72c5";
  bool _isUpdating = false;

  /// 注册更新配置
  /// [updateConfig] 实现更新检测组件
  void registerConfig<C extends OnAppUpdate>(C updateConfig) {
    ConfigManager.instance.addConfig(_updateConfigKey, updateConfig);
  }

  OnAppUpdate? _getUpdateConfig() {
    var config = ConfigManager.instance.getConfig(_updateConfigKey);
    if (config is! OnAppUpdate) {
      return null;
    }
    return config;
  }

  /// 检测与更新版本
  /// 过滤掉10秒内频繁调用
  void detection() async {
    if (_isUpdating) {
      return;
    }
    _isUpdating = true;
    var time = MemoryUtils.instance.getInt(_timeCacheKey);
    var diff = System.currentTimeMillis - time;
    if (diff <= 10000) {
      _isUpdating = false;
      return;
    }
    MemoryUtils.instance.set(_timeCacheKey, System.currentTimeMillis);
    //获取应用信息
    var appInfo = await EyFlutter.instance.getAppInfo();
    var updateVersion = await _getUpdateVersion(appInfo);
    if (updateVersion.packageName.isEmptyString) {
      _isUpdating = false;
      return;
    }
    if (updateVersion.enableLowBuild) {
      if (updateVersion.lowBuild > appInfo.build) {
        _isUpdating = false;
        var updateConfig = _getUpdateConfig();
        updateConfig?.updateCall(isForceUpdate: true, info: updateVersion);
        return;
      }
    }
    if (updateVersion.build > appInfo.build) {
      _isUpdating = false;
      var updateConfig = _getUpdateConfig();
      updateConfig?.updateCall(isForceUpdate: false, info: updateVersion);
      return;
    }
    _isUpdating = false;
  }

  Future<VersionInfo> _getUpdateVersion(ApplicationInfo appInfo) async {
    var url = await ConfigTable.instance.getValue(configKey: "_APP_UPDATE_URL");
    if (url.isEmptyString) {
      return Future.value(VersionInfo());
    }
    var deviceTag = "";
    if (Platform.isAndroid) {
      deviceTag = await ConfigTable.instance.getValue(configKey: "_ANDROID_DEVICE_TAG");
    } else if (Platform.isIOS) {
      deviceTag = await ConfigTable.instance.getValue(configKey: "_IOS_DEVICE_TAG");
    }
    if (deviceTag.isEmptyString) {
      return Future.value(VersionInfo());
    }
    Retrofit retrofit = Retrofit(api: url, method: Method.GET);
    Map<String, dynamic> dataMap = await DioManager.instance.asyncRequest(retrofit: retrofit, params: {
      "packageName": appInfo.packageName,
      "deviceTag": deviceTag,
    });
    var data = dataMap["data"];
    if (data is! Map<String, dynamic>) {
      return Future.value(VersionInfo());
    }
    return Future.value(VersionInfo.fromJson(data));
  }
}
